use criterion::{criterion_group, criterion_main, Benchmark, Criterion};
use metrics::{Key, KeyData, Label, NoopRecorder, Recorder, SharedString};
use metrics_util::layers::{FilterLayer, Layer};

fn layer_benchmark(c: &mut Criterion) {
    c.bench(
        "filter",
        Benchmark::new("match", |b| {
            let patterns = vec!["tokio"];
            let filter_layer = FilterLayer::from_patterns(patterns);
            let recorder = filter_layer.layer(NoopRecorder);
            static KEY_NAME: [SharedString; 1] = [SharedString::const_str("tokio.foo")];
            static KEY_LABELS: [Label; 1] = [Label::from_static_parts("foo", "bar")];
            static KEY_DATA: KeyData = KeyData::from_static_parts(&KEY_NAME, &KEY_LABELS);

            b.iter(|| {
                recorder.increment_counter(Key::Borrowed(&KEY_DATA), 1);
            })
        })
        .with_function("no match", |b| {
            let patterns = vec!["tokio"];
            let filter_layer = FilterLayer::from_patterns(patterns);
            let recorder = filter_layer.layer(NoopRecorder);
            static KEY_NAME: [SharedString; 1] = [SharedString::const_str("hyper.foo")];
            static KEY_LABELS: [Label; 1] = [Label::from_static_parts("foo", "bar")];
            static KEY_DATA: KeyData = KeyData::from_static_parts(&KEY_NAME, &KEY_LABELS);

            b.iter(|| {
                recorder.increment_counter(Key::Borrowed(&KEY_DATA), 1);
            })
        })
        .with_function("deep match", |b| {
            let patterns = vec!["tokio"];
            let filter_layer = FilterLayer::from_patterns(patterns);
            let recorder = filter_layer.layer(NoopRecorder);
            static KEY_NAME: [SharedString; 2] = [
                SharedString::const_str("prefix"),
                SharedString::const_str("tokio.foo"),
            ];
            static KEY_LABELS: [Label; 1] = [Label::from_static_parts("foo", "bar")];
            static KEY_DATA: KeyData = KeyData::from_static_parts(&KEY_NAME, &KEY_LABELS);

            b.iter(|| {
                recorder.increment_counter(Key::Borrowed(&KEY_DATA), 1);
            })
        })
        .with_function("noop recorder overhead (increment_counter)", |b| {
            let recorder = NoopRecorder;
            static KEY_NAME: [SharedString; 1] = [SharedString::const_str("tokio.foo")];
            static KEY_LABELS: [Label; 1] = [Label::from_static_parts("foo", "bar")];
            static KEY_DATA: KeyData = KeyData::from_static_parts(&KEY_NAME, &KEY_LABELS);

            b.iter(|| {
                recorder.increment_counter(Key::Borrowed(&KEY_DATA), 1);
            })
        }),
    );
}

criterion_group!(benches, layer_benchmark);
criterion_main!(benches);
